// SerialMonitorDlg.cpp : implementation file
//

#include "stdafx.h"
#include "SerialMonitor.h"
#include "SerialMonitorDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSerialMonitorDlg dialog




CSerialMonitorDlg::CSerialMonitorDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSerialMonitorDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CSerialMonitorDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_DEVICES, lvDevices);
	DDX_Control(pDX, IDC_LIST_EVENTS, lbEvents);
}

BEGIN_MESSAGE_MAP(CSerialMonitorDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON_CLEAR, &CSerialMonitorDlg::OnBnClickedButtonClear)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_BUTTON_ENUM, &CSerialMonitorDlg::OnBnClickedButtonEnum)
	ON_BN_CLICKED(IDC_BUTTON_START, &CSerialMonitorDlg::OnBnClickedButtonStart)
	ON_BN_CLICKED(IDC_BUTTON_STOP, &CSerialMonitorDlg::OnBnClickedButtonStop)
END_MESSAGE_MAP()


// CSerialMonitorDlg message handlers

BOOL CSerialMonitorDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here

	lvDevices.InsertColumn(0, _T("Friendly name"), 0, 390);
	lvDevices.InsertColumn(1, _T("Is modem"), 0, 80);
	lvDevices.InsertColumn(2, _T("Device name"), 0, 80);
	lvDevices.SetExtendedStyle(lvDevices.GetExtendedStyle() | LVS_EX_FULLROWSELECT);

	__hook(&CwclSerialMonitor::OnInserted, &wclSerialMonitor, &CSerialMonitorDlg::wclSerialMonitorInserted);
	__hook(&CwclSerialMonitor::OnRemoved, &wclSerialMonitor, &CSerialMonitorDlg::wclSerialMonitorRemoved);

	__hook(&CwclPowerEventsMonitor::OnPowerStateChanged, &FPowerMonitor, &CSerialMonitorDlg::PowerStateChanged);

	FPowerMonitor.Open();

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSerialMonitorDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSerialMonitorDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void CSerialMonitorDlg::OnBnClickedButtonClear()
{
	lbEvents.ResetContent();
}

void CSerialMonitorDlg::OnDestroy()
{
	CDialog::OnDestroy();

	wclSerialMonitor.Stop();

	FPowerMonitor.Close();

	__unhook(&wclSerialMonitor);
	__unhook(&FPowerMonitor);
}

void CSerialMonitorDlg::OnBnClickedButtonEnum()
{
	lvDevices.DeleteAllItems();
	
	wclSerialDevices Devices;
	int Res = wclSerialMonitor.EnumDevices(Devices);
	CString s;
	s.Format(_T("Enumerate serial devices: 0x%.8X"), Res);
	lbEvents.AddString(s);
	
	if (Res == WCL_E_SUCCESS)
	{
		int j = 0;
		for (wclSerialDevices::iterator i = Devices.begin(); i != Devices.end(); i++)
		{
			lvDevices.InsertItem(j, (*i).FriendlyName.c_str());
			if ((*i).IsModem)
				s = _T("True");
			else
				s = _T("False");
			lvDevices.SetItemText(j, 1, s);
			lvDevices.SetItemText(j, 2, (*i).DeviceName.c_str());
			j++;
		}
	}
}

void CSerialMonitorDlg::OnBnClickedButtonStart()
{
	int Res = wclSerialMonitor.Start();
	CString s;
	s.Format(_T("Monitoring started: 0x%.8X"), Res);
	lbEvents.AddString(s);
}

void CSerialMonitorDlg::OnBnClickedButtonStop()
{
	int Res = wclSerialMonitor.Stop();
	CString s;
	s.Format(_T("Monitoring stopped: 0x%.8X"), Res);
	lbEvents.AddString(s);
}

void CSerialMonitorDlg::wclSerialMonitorInserted(void* Sender,
	const wclSerialDevice& Device)
{
	UNREFERENCED_PARAMETER(Sender);

	lbEvents.AddString(_T("Device inserted:"));
	CString s(Device.FriendlyName.c_str());
	lbEvents.AddString(_T("  Friendly name: ") + s);
	if (Device.IsModem)
		lbEvents.AddString(_T("  Modem: True"));
	else
		lbEvents.AddString(_T("  Modem: False"));
	s = CString(Device.DeviceName.c_str());
	lbEvents.AddString(_T("  DeviceName: ") + s);
}

void CSerialMonitorDlg::wclSerialMonitorRemoved(void* Sender,
	const wclSerialDevice& Device)
{
	UNREFERENCED_PARAMETER(Sender);

	lbEvents.AddString(_T("Device removed:"));
	CString s(Device.FriendlyName.c_str());
	lbEvents.AddString(_T("  Friendly name: ") + s);
	if (Device.IsModem)
		lbEvents.AddString(_T("  Modem: True"));
	else
		lbEvents.AddString(_T("  Modem: False"));
	s = CString(Device.DeviceName.c_str());
	lbEvents.AddString(_T("  DeviceName: ") + s);
}

void CSerialMonitorDlg::PowerStateChanged(void* Sender, const wclPowerState State)
{
	UNREFERENCED_PARAMETER(Sender);

	static bool Running = false;

	switch (State)
	{
		case psResumeAutomatic:
			if (Running)
			{
				Running = false;
				wclSerialMonitor.Start();
			}
			lbEvents.AddString(_T("Power changed: psResumeAutomatic"));
			break;

		case psResume:
			lbEvents.AddString(_T("Power changed: psResume"));
			break;

		case psSuspend:
			if (wclSerialMonitor.Monitoring)
			{
				Running = true;
				wclSerialMonitor.Stop();
			}
			lbEvents.AddString(_T("Power changed: psSuspend"));
			break;

		case psUnknown:
			lbEvents.AddString(_T("Power changed: psUnknown"));
			break;
	}
}