// UsbMonitorDlg.cpp : implementation file
//

#include "stdafx.h"
#include "UsbMonitor.h"
#include "UsbMonitorDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CUsbMonitorDlg dialog




CUsbMonitorDlg::CUsbMonitorDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CUsbMonitorDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CUsbMonitorDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_DEVICES, lvDevices);
	DDX_Control(pDX, IDC_LIST_EVENTS, lbEvents);
}

BEGIN_MESSAGE_MAP(CUsbMonitorDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON_CLEAR, &CUsbMonitorDlg::OnBnClickedButtonClear)
	ON_BN_CLICKED(IDC_BUTTON_ENUMERATE, &CUsbMonitorDlg::OnBnClickedButtonEnumerate)
	ON_BN_CLICKED(IDC_BUTTON_START, &CUsbMonitorDlg::OnBnClickedButtonStart)
	ON_BN_CLICKED(IDC_BUTTON_STOP, &CUsbMonitorDlg::OnBnClickedButtonStop)
	ON_BN_CLICKED(IDC_BUTTON_ENABLE, &CUsbMonitorDlg::OnBnClickedButtonEnable)
	ON_BN_CLICKED(IDC_BUTTON_DISABLE, &CUsbMonitorDlg::OnBnClickedButtonDisable)
	ON_WM_DESTROY()
END_MESSAGE_MAP()


// CUsbMonitorDlg message handlers

BOOL CUsbMonitorDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	lvDevices.InsertColumn(0, _T("Friendly name"), 0, 150);
	lvDevices.InsertColumn(1, _T("Hardware ID"), 0, 150);
	lvDevices.InsertColumn(2, _T("VID"), 0, 60);
	lvDevices.InsertColumn(3, _T("PID"), 0, 60);
	lvDevices.InsertColumn(4, _T("REV"), 0, 60);
	lvDevices.InsertColumn(5, _T("Manufacturer"), 0, 150);
	lvDevices.InsertColumn(6, _T("Instance"), 0, 150);
	lvDevices.InsertColumn(7, _T("Class GUID"), 0, 250);
	lvDevices.InsertColumn(8, _T("Enabled "), 0, 50);
	lvDevices.SetExtendedStyle(lvDevices.GetExtendedStyle() | LVS_EX_FULLROWSELECT);

	__hook(&CwclUsbMonitor::OnInserted, &Monitor, &CUsbMonitorDlg::Inserted);
	__hook(&CwclUsbMonitor::OnRemoved, &Monitor, &CUsbMonitorDlg::Removed);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CUsbMonitorDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CUsbMonitorDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CUsbMonitorDlg::OnBnClickedButtonClear()
{
	lbEvents.ResetContent();
}

CString IntToHex(const unsigned short i)
{
	CString s;
	s.Format(_T("%.4X"), i);
	return s;
}

void CUsbMonitorDlg::OnBnClickedButtonEnumerate()
{
	lvDevices.DeleteAllItems();

	wclUsbDevices Devices;
	int Res = Monitor.EnumDevices(Devices);
	if (Res != WCL_E_SUCCESS)
	{
		CString s;
		s.Format(_T("Enumeration failed: 0x%.8X"), Res);
		AfxMessageBox(s);
	}
	else
	{
		for (wclUsbDevices::iterator i = Devices.begin(); i != Devices.end(); i++)
		{
			wclUsbDevice Device = (*i);
			int n = lvDevices.GetItemCount();
			
			LPOLESTR Guid;
			StringFromCLSID((*i).ClassGuid, &Guid);

			lvDevices.InsertItem(n, Device.FriendlyName.c_str());
			lvDevices.SetItemText(n, 1, Device.HardwareId.c_str());
			lvDevices.SetItemText(n, 2, IntToHex(Device.VendorId));
			lvDevices.SetItemText(n, 3, IntToHex(Device.ProductId));
			lvDevices.SetItemText(n, 4, IntToHex(Device.Revision));
			lvDevices.SetItemText(n, 5, Device.Manufacturer.c_str());
			lvDevices.SetItemText(n, 6, Device.Instance.c_str());
			lvDevices.SetItemText(n, 7, Guid);
			if (Device.Enabled)
				lvDevices.SetItemText(n, 8, _T("True"));
			else
				lvDevices.SetItemText(n, 8, _T("False"));

			CoTaskMemFree(Guid);
		}
	}
}

void CUsbMonitorDlg::OnBnClickedButtonStart()
{
	int Res = Monitor.Start();
	if (Res != WCL_E_SUCCESS)
	{
		CString s;
		s.Format(_T("Start monitoring failed: 0x%.8X"), Res);
		AfxMessageBox(s);
	}
	else
		lbEvents.AddString(_T("Monitoring started"));
}

void CUsbMonitorDlg::OnBnClickedButtonStop()
{
	int Res = Monitor.Stop();
	if (Res != WCL_E_SUCCESS)
	{
		CString s;
		s.Format(_T("Stop monitoring failed: 0x%.8X"), Res);
		AfxMessageBox(s);
	}
	else
		lbEvents.AddString(_T("Monitoring stopped"));
}

void CUsbMonitorDlg::Inserted(void* Sender, const wclUsbDevice& Device)
{
	UNREFERENCED_PARAMETER(Sender);

	lbEvents.AddString(_T("Device inserted: ") + CString(Device.Instance.c_str()));
}

void CUsbMonitorDlg::Removed(void* Sender, const wclUsbDevice& Device)
{
	UNREFERENCED_PARAMETER(Sender);

	lbEvents.AddString(_T("Device removed: ") + CString(Device.Instance.c_str()));
}

void CUsbMonitorDlg::OnBnClickedButtonEnable()
{
	POSITION Pos = lvDevices.GetFirstSelectedItemPosition();
	if (Pos == NULL)
		AfxMessageBox(_T("Select device"));
	else
	{
		CString Inst = lvDevices.GetItemText(lvDevices.GetNextSelectedItem(Pos), 3);
		int Res = Monitor.Enable(tstring(Inst.GetBuffer()));
		if (Res != WCL_E_SUCCESS)
		{
			Inst.Format(_T("Enabling device failed: 0x%.8X"), Res);
			AfxMessageBox(Inst);
		}
	}
}

void CUsbMonitorDlg::OnBnClickedButtonDisable()
{
	POSITION Pos = lvDevices.GetFirstSelectedItemPosition();
	if (Pos == NULL)
		AfxMessageBox(_T("Select device"));
	else
	{
		CString Inst = lvDevices.GetItemText(lvDevices.GetNextSelectedItem(Pos), 3);
		int Res = Monitor.Disable(tstring(Inst.GetBuffer()));
		if (Res != WCL_E_SUCCESS)
		{
			Inst.Format(_T("Disabling device failed: 0x%.8X"), Res);
			AfxMessageBox(Inst);
		}
	}
}

void CUsbMonitorDlg::OnDestroy()
{
	CDialog::OnDestroy();

	// TODO: Add your message handler code here

	Monitor.Stop();
	__unhook(&Monitor);
}
