// ConsoleApplication1.cpp : Defines the entry point for the console application.
//

#include "pch.h"
#include <process.h>

#include "..\..\..\..\Source\Serial\wclSerialDevices.h"

using namespace std;
using namespace wclCommon;
using namespace wclSerial;

class CSerialMonitor
{
private:
	HANDLE	TerminateEvent;
	HANDLE	Thread;

	string wtoa(const tstring& w) const
	{
		return string(w.begin(), w.end());
	}

	string btoa(const bool b)
	{
		if (b)
			return "true";
		return "false";
	}
	
	void Monitor_OnRemoved(void* Sender, const wclSerialDevice& Device)
	{
		UNREFERENCED_PARAMETER(Sender);

		cout << "Device REMOVED: " << endl;
		cout << "  Friendly name: " << wtoa(Device.FriendlyName).c_str() << endl;
		cout << "  Is modem: " << btoa(Device.IsModem) << endl;
		cout << "  Name: " << wtoa(Device.DeviceName).c_str() << endl;
	}

	void Monitor_OnInserted(void* Sender, const wclSerialDevice& Device)
	{
		UNREFERENCED_PARAMETER(Sender);

		cout << "Device INSERTED: " << endl;
		cout << "  Friendly name: " << wtoa(Device.FriendlyName).c_str() << endl;
		cout << "  Is modem: " << btoa(Device.IsModem) << endl;
		cout << "  Name: " << wtoa(Device.DeviceName).c_str() << endl;
	}

	static UINT __stdcall _ThreadProc(LPVOID Param)
	{
		((CSerialMonitor*)Param)->ThreadProc();
		return 0;
	}

	void ThreadProc()
	{
		CwclSerialMonitor* Monitor = new CwclSerialMonitor();
		__hook(&CwclSerialMonitor::OnInserted, Monitor, &CSerialMonitor::Monitor_OnInserted);
		__hook(&CwclSerialMonitor::OnRemoved, Monitor, &CSerialMonitor::Monitor_OnRemoved);

		cout << "Enumerate installed device." << endl;
		wclSerialDevices Devices;
		int Res = Monitor->EnumDevices(Devices);
		if (Res != WCL_E_SUCCESS)
			cout << "Failed to enumerate serial devices: 0x" << hex << Res << dec << endl;
		else
		{
			if (Devices.size() == 0)
				cout << "No serial devices were found" << endl;
			else
			{
				int n = 0;
				for (wclSerialDevices::iterator i = Devices.begin(); i != Devices.end(); i++)
				{
					cout << "Device " << n << endl;
					cout << "  Friendly name: " << wtoa((*i).FriendlyName).c_str() << endl;
					cout << "  Is modem: " << btoa((*i).IsModem) << endl;
					cout << "  Name: " << wtoa((*i).DeviceName).c_str() << endl;
					n++;
				}
			}
		}

		cout << "Try to start monitoring" << endl;
		Res = Monitor->Start();
		if (Res != WCL_E_SUCCESS)
			cout << "Failed to start monitoring serial devices 0x" << hex << Res << dec << endl;
		else
			cout << "Monitoring started";

		CwclMessageBroadcaster::Wait(TerminateEvent);

		if (Monitor->Monitoring)
		{
			cout << "Try to stop monuioring" << endl;
			Res = Monitor->Stop();
			if (Res != WCL_E_SUCCESS)
				cout << "Failed to stop monitoring serial devices 0x" << hex << Res << dec << endl;
			else
				cout << "Monitoring stopped" << endl;
		}
		__unhook(Monitor);
		delete Monitor;
	}

public:
	CSerialMonitor()
	{
		TerminateEvent = NULL;
		Thread = NULL;
	}

	void Start()
	{
		TerminateEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
		Thread = (HANDLE)_beginthreadex(NULL, 0, _ThreadProc, this, 0, NULL);
	}

	void Stop()
	{
		SetEvent(TerminateEvent);
		WaitForSingleObject(Thread, INFINITE);
		CloseHandle(Thread);

		CloseHandle(TerminateEvent);
	}
};

int main(int argc, char **argv)
{
	UNREFERENCED_PARAMETER(argc);
	UNREFERENCED_PARAMETER(argv);

	CwclMessageBroadcaster::SetSyncMethod(skApc);
	
	system("cls");
	cout << "SERIAL MONITOR DEMO" << endl;
	cout << "===================" << endl;
	cout << "The demo shows how to enumerate and monitor serial devices." << endl;

	CSerialMonitor* Monitor = new CSerialMonitor();
	Monitor->Start();
	system("pause");
	Monitor->Stop();
	delete Monitor;
}